@@ -2,17 +2,21 @@ from collections import defaultdict
 from numbers import Integral
 
 import pandas as pd
+from pandas.api.types import is_scalar
 from tlz import partition_all
 
-from ..base import compute_as_if_collection, tokenize
-from ..utils import Dispatch
-from . import methods
-from .accessor import Accessor
-from .utils import (
+from dask.base import compute_as_if_collection, tokenize
+from dask.dataframe import methods
+from dask.dataframe.accessor import Accessor
+from dask.dataframe.dispatch import (  # noqa: F401
+    categorical_dtype,
+    categorical_dtype_dispatch,
+    is_categorical_dtype,
+)
+from dask.dataframe.utils import (
+    AttributeNotImplementedError,
     clear_known_categories,
     has_known_categories,
-    is_categorical_dtype,
-    is_scalar,
 )
 
 
@@ -47,7 +51,7 @@ def _get_categories(df, columns, index):
     for col in columns:
         x = df[col]
         if is_categorical_dtype(x):
-            res[col] = pd.Series(x.cat.categories)
+            res[col] = x._constructor(x.cat.categories)
         else:
             res[col] = x.dropna().drop_duplicates()
     if index:
@@ -145,7 +149,9 @@ def categorize(df, columns=None, index=None, split_every=None, **kwargs):
     dsk.update(df.dask)
 
     # Compute the categories
-    categories, index = compute_as_if_collection(type(df), dsk, (prefix, 0), **kwargs)
+    categories, index = compute_as_if_collection(
+        df.__class__, dsk, (prefix, 0), **kwargs
+    )
 
     # Categorize each partition
     return df.map_partitions(_categorize_block, categories, index)
@@ -176,6 +182,16 @@ class CategoricalAccessor(Accessor):
     """
 
     _accessor_name = "cat"
+    _accessor_methods = (
+        "add_categories",
+        "as_ordered",
+        "as_unordered",
+        "remove_categories",
+        "rename_categories",
+        "reorder_categories",
+        "set_categories",
+    )
+    _accessor_properties = ()
 
     @property
     def known(self):
@@ -209,6 +225,7 @@ class CategoricalAccessor(Accessor):
 
     @property
     def ordered(self):
+        """Whether the categories have an ordered relationship"""
         return self._delegate_property(self._series._meta, "cat", "ordered")
 
     @property
@@ -222,7 +239,7 @@ class CategoricalAccessor(Accessor):
                 "supported.  Please use `column.cat.as_known()` or "
                 "`df.categorize()` beforehand to ensure known categories"
             )
-            raise NotImplementedError(msg)
+            raise AttributeNotImplementedError(msg)
         return self._delegate_property(self._series._meta, "cat", "categories")
 
     @property
@@ -236,7 +253,7 @@ class CategoricalAccessor(Accessor):
                 "supported.  Please use `column.cat.as_known()` or "
                 "`df.categorize()` beforehand to ensure known categories"
             )
-            raise NotImplementedError(msg)
+            raise AttributeNotImplementedError(msg)
         return self._property_map("codes")
 
     def remove_unused_categories(self):
@@ -274,16 +291,3 @@ class CategoricalAccessor(Accessor):
             meta=meta,
             token="cat-set_categories",
         )
-
-
-categorical_dtype_dispatch = Dispatch("CategoricalDtype")
-
-
-def categorical_dtype(meta, categories=None, ordered=False):
-    func = categorical_dtype_dispatch.dispatch(type(meta))
-    return func(categories=categories, ordered=ordered)
-
-
-@categorical_dtype_dispatch.register((pd.DataFrame, pd.Series, pd.Index))
-def categorical_dtype_pandas(categories=None, ordered=False):
-    return pd.api.types.CategoricalDtype(categories=categories, ordered=ordered)
