@@ -11,23 +11,28 @@ from numbers import Number
 
 import numpy as np
 import pandas as pd
-from pandas.api.types import is_scalar  # noqa: F401
 from pandas.api.types import is_categorical_dtype, is_dtype_equal
 
-from ..base import get_scheduler, is_dask_collection
-from ..core import get_deps
-from ..utils import is_arraylike  # noqa: F401
-from ..utils import asciitable
-from ..utils import is_dataframe_like as dask_is_dataframe_like
-from ..utils import is_index_like as dask_is_index_like
-from ..utils import is_series_like as dask_is_series_like
-from ..utils import typename
-from . import _dtypes  # noqa: F401 register pandas extension types
-from . import methods
-from ._compat import PANDAS_GT_110, PANDAS_GT_120, tm  # noqa: F401
-from .dispatch import make_meta  # noqa : F401
-from .dispatch import make_meta_obj, meta_nonempty  # noqa : F401
-from .extensions import make_scalar
+from dask.base import get_scheduler, is_dask_collection
+from dask.core import get_deps
+from dask.dataframe import (  # noqa: F401 register pandas extension types
+    _dtypes,
+    methods,
+)
+from dask.dataframe._compat import PANDAS_GT_110, PANDAS_GT_120, tm  # noqa: F401
+from dask.dataframe.dispatch import (  # noqa : F401
+    make_meta,
+    make_meta_obj,
+    meta_nonempty,
+)
+from dask.dataframe.extensions import make_scalar
+from dask.utils import (
+    asciitable,
+    is_dataframe_like,
+    is_index_like,
+    is_series_like,
+    typename,
+)
 
 meta_object_types: tuple[type, ...] = (pd.Series, pd.DataFrame, pd.Index, pd.MultiIndex)
 try:
@@ -321,18 +326,6 @@ def _nonempty_scalar(x):
     raise TypeError(f"Can't handle meta of type '{typename(type(x))}'")
 
 
-def is_dataframe_like(df):
-    return dask_is_dataframe_like(df)
-
-
-def is_series_like(s):
-    return dask_is_series_like(s)
-
-
-def is_index_like(s):
-    return dask_is_index_like(s)
-
-
 def check_meta(x, meta, funcname=None, numeric_equal=True):
     """Check that the dask metadata matches the result.
 
@@ -530,6 +523,7 @@ def assert_eq(
     check_dtype=True,
     check_divisions=True,
     check_index=True,
+    sort_results=True,
     scheduler="sync",
     **kwargs,
 ):
@@ -552,7 +546,7 @@ def assert_eq(
         a = a.to_pandas()
     if hasattr(b, "to_pandas"):
         b = b.to_pandas()
-    if isinstance(a, (pd.DataFrame, pd.Series)):
+    if isinstance(a, (pd.DataFrame, pd.Series)) and sort_results:
         a = _maybe_sort(a, check_index)
         b = _maybe_sort(b, check_index)
     if not check_index:
@@ -720,3 +714,7 @@ def drop_by_shallow_copy(df, columns, errors="raise"):
         columns = [columns]
     df2.drop(columns=columns, inplace=True, errors=errors)
     return df2
+
+
+class AttributeNotImplementedError(NotImplementedError, AttributeError):
+    """NotImplementedError and AttributeError"""
