@@ -8,10 +8,10 @@ import sys
 from functools import partial
 from operator import add
 
-from distributed.utils_test import client as c  # noqa F401
+from distributed.utils_test import cleanup  # noqa F401
 from distributed.utils_test import cluster_fixture  # noqa F401
-from distributed.utils_test import loop  # noqa F401
-from distributed.utils_test import cluster, gen_cluster, inc, varying
+from distributed.utils_test import client as c  # noqa F401
+from distributed.utils_test import cluster, gen_cluster, inc, loop, varying  # noqa F401
 
 import dask
 import dask.bag as db
@@ -375,11 +375,6 @@ def test_blockwise_dataframe_io(c, tmpdir, io, fuse, from_futures):
     pd = pytest.importorskip("pandas")
     dd = pytest.importorskip("dask.dataframe")
 
-    # TODO: this configuration is flaky on osx in CI
-    # See https://github.com/dask/dask/issues/8816
-    if from_futures and sys.platform == "darwin":
-        pytest.xfail("This test sometimes fails on osx in CI")
-
     df = pd.DataFrame({"x": [1, 2, 3] * 5, "y": range(15)})
 
     if from_futures:
@@ -830,3 +825,21 @@ async def test_gh_8991(c, s, a, b):
 
     # The recalculated dependencies should still be the same!
     assert deps == deps_nokeys
+
+
+def test_parquet_processes_false(tmpdir):
+    # Check dask#8581 Parquet work-around
+    # See: https://github.com/dask/dask/pull/9015
+    dd = pytest.importorskip("dask.dataframe")
+
+    with distributed.LocalCluster(
+        scheduler_port=0,
+        dashboard_address=":0",
+        asynchronous=False,
+        n_workers=1,
+        nthreads=1,
+        processes=False,
+    ) as cluster:
+        with distributed.Client(cluster, asynchronous=False):
+            dd.DataFrame.from_dict({"a": range(10)}, npartitions=2).to_parquet(tmpdir)
+            dd.read_parquet(tmpdir).compute()
